#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

import json

import pytest

from airflow.models import Connection
from airflow.providers.dingding.hooks.dingding import DingdingHook


class TestDingdingHook:
    conn_id = "dingding_conn_id_test"

    @pytest.fixture(autouse=True)
    def setup_connections(self, create_connection_without_db):
        create_connection_without_db(
            Connection(
                conn_id=self.conn_id,
                conn_type="dingding",
                host="https://oapi.dingtalk.com",
                password="you_token_here",
            )
        )

    def test_get_endpoint_conn_id(self):
        hook = DingdingHook(dingding_conn_id=self.conn_id)
        endpoint = hook._get_endpoint()
        assert endpoint == "robot/send?access_token=you_token_here"

    def test_build_text_message_not_remind(self):
        config = {
            "dingding_conn_id": self.conn_id,
            "message_type": "text",
            "message": "Airflow dingding text message remind no one",
            "at_mobiles": False,
            "at_all": False,
        }
        expect = {
            "msgtype": "text",
            "text": {"content": "Airflow dingding text message remind no one"},
            "at": {"atMobiles": False, "isAtAll": False},
        }
        hook = DingdingHook(**config)
        message = hook._build_message()
        assert json.dumps(expect) == message

    def test_build_text_message_remind_specific(self):
        config = {
            "dingding_conn_id": self.conn_id,
            "message_type": "text",
            "message": "Airflow dingding text message remind specific users",
            "at_mobiles": ["1234", "5768"],
            "at_all": False,
        }
        expect = {
            "msgtype": "text",
            "text": {"content": "Airflow dingding text message remind specific users"},
            "at": {"atMobiles": ["1234", "5768"], "isAtAll": False},
        }
        hook = DingdingHook(**config)
        message = hook._build_message()
        assert json.dumps(expect) == message

    def test_build_text_message_remind_all(self):
        config = {
            "dingding_conn_id": self.conn_id,
            "message_type": "text",
            "message": "Airflow dingding text message remind all user in group",
            "at_all": True,
        }
        expect = {
            "msgtype": "text",
            "text": {"content": "Airflow dingding text message remind all user in group"},
            "at": {"atMobiles": None, "isAtAll": True},
        }
        hook = DingdingHook(**config)
        message = hook._build_message()
        assert json.dumps(expect) == message

    def test_build_markdown_message_remind_specific(self):
        msg = {
            "title": "Airflow dingding markdown message",
            "text": "# Markdown message title\ncontent content .. \n### sub-title\n"
            "![logo](https://airflow.apache.org/_images/pin_large.png)",
        }
        config = {
            "dingding_conn_id": self.conn_id,
            "message_type": "markdown",
            "message": msg,
            "at_mobiles": ["1234", "5678"],
            "at_all": False,
        }
        expect = {
            "msgtype": "markdown",
            "markdown": msg,
            "at": {"atMobiles": ["1234", "5678"], "isAtAll": False},
        }
        hook = DingdingHook(**config)
        message = hook._build_message()
        assert json.dumps(expect) == message

    def test_build_markdown_message_remind_all(self):
        msg = {
            "title": "Airflow dingding markdown message",
            "text": "# Markdown message title\ncontent content .. \n### sub-title\n"
            "![logo](https://airflow.apache.org/_images/pin_large.png)",
        }
        config = {
            "dingding_conn_id": self.conn_id,
            "message_type": "markdown",
            "message": msg,
            "at_all": True,
        }
        expect = {"msgtype": "markdown", "markdown": msg, "at": {"atMobiles": None, "isAtAll": True}}
        hook = DingdingHook(**config)
        message = hook._build_message()
        assert json.dumps(expect) == message

    def test_build_link_message(self):
        msg = {
            "title": "Airflow dingding link message",
            "text": "Airflow official documentation link",
            "messageUrl": "https://airflow.apache.org",
            "picURL": "https://airflow.apache.org/_images/pin_large.png",
        }
        config = {"dingding_conn_id": self.conn_id, "message_type": "link", "message": msg}
        expect = {"msgtype": "link", "link": msg}
        hook = DingdingHook(**config)
        message = hook._build_message()
        assert json.dumps(expect) == message

    def test_build_single_action_card_message(self):
        msg = {
            "title": "Airflow dingding single actionCard message",
            "text": "Airflow dingding single actionCard message\n"
            "![logo](https://airflow.apache.org/_images/pin_large.png)\n"
            "This is a official logo in Airflow website.",
            "hideAvatar": "0",
            "btnOrientation": "0",
            "singleTitle": "read more",
            "singleURL": "https://airflow.apache.org",
        }
        config = {"dingding_conn_id": self.conn_id, "message_type": "actionCard", "message": msg}
        expect = {"msgtype": "actionCard", "actionCard": msg}
        hook = DingdingHook(**config)
        message = hook._build_message()
        assert json.dumps(expect) == message

    def test_build_multi_action_card_message(self):
        msg = {
            "title": "Airflow dingding multi actionCard message",
            "text": "Airflow dingding multi actionCard message\n"
            "![logo](https://airflow.apache.org/_images/pin_large.png)\n"
            "Airflow documentation and GitHub",
            "hideAvatar": "0",
            "btnOrientation": "0",
            "btns": [
                {"title": "Airflow Documentation", "actionURL": "https://airflow.apache.org"},
                {"title": "Airflow GitHub", "actionURL": "https://github.com/apache/airflow"},
            ],
        }
        config = {"dingding_conn_id": self.conn_id, "message_type": "actionCard", "message": msg}
        expect = {"msgtype": "actionCard", "actionCard": msg}
        hook = DingdingHook(**config)
        message = hook._build_message()
        assert json.dumps(expect) == message

    def test_build_feed_card_message(self):
        msg = {
            "links": [
                {
                    "title": "Airflow DAG feed card",
                    "messageURL": "https://airflow.apache.org/docs/apache-airflow/stable/ui.html",
                    "picURL": "https://airflow.apache.org/_images/dags.png",
                },
                {
                    "title": "Airflow grid feed card",
                    "messageURL": "https://airflow.apache.org/docs/apache-airflow/stable/ui.html",
                    "picURL": "https://airflow.apache.org/_images/grid.png",
                },
                {
                    "title": "Airflow graph feed card",
                    "messageURL": "https://airflow.apache.org/docs/apache-airflow/stable/ui.html",
                    "picURL": "https://airflow.apache.org/_images/graph.png",
                },
            ]
        }
        config = {"dingding_conn_id": self.conn_id, "message_type": "feedCard", "message": msg}
        expect = {"msgtype": "feedCard", "feedCard": msg}
        hook = DingdingHook(**config)
        message = hook._build_message()
        assert json.dumps(expect) == message

    def test_send_not_support_type(self):
        config = {
            "dingding_conn_id": self.conn_id,
            "message_type": "not_support_type",
            "message": "Airflow dingding text message remind no one",
        }
        hook = DingdingHook(**config)
        with pytest.raises(ValueError, match="receive not_support_type"):
            hook.send()



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
"""
This is an example dag for using the DingdingOperator.
"""

from __future__ import annotations

import os
from datetime import datetime, timedelta

from airflow import DAG
from airflow.providers.dingding.hooks.dingding import DingdingHook
from airflow.providers.dingding.operators.dingding import DingdingOperator

ENV_ID = os.environ.get("SYSTEM_TESTS_ENV_ID")
DAG_ID = "example_dingding_operator"


# [START howto_operator_dingding_failure_callback]
def failure_callback(context):
    """
    The function that will be executed on failure.

    :param context: The context of the executed task.
    """
    message = f"The task {context['ti'].task_id} failed"
    DingdingHook(message_type="text", message=message, at_all=True).send()


# [END howto_operator_dingding_failure_callback]

with DAG(
    dag_id=DAG_ID,
    default_args={"retries": 3, "on_failure_callback": failure_callback},
    schedule="@once",
    dagrun_timeout=timedelta(minutes=60),
    start_date=datetime(2021, 1, 1),
    tags=["example"],
    catchup=False,
) as dag:
    # [START howto_operator_dingding]
    text_msg_remind_none = DingdingOperator(
        task_id="text_msg_remind_none",
        message_type="text",
        message="Airflow dingding text message remind none",
        at_mobiles=None,
        at_all=False,
    )
    # [END howto_operator_dingding]

    text_msg_remind_specific = DingdingOperator(
        task_id="text_msg_remind_specific",
        message_type="text",
        message="Airflow dingding text message remind specific users",
        at_mobiles=["156XXXXXXXX", "130XXXXXXXX"],
        at_all=False,
    )

    text_msg_remind_include_invalid = DingdingOperator(
        task_id="text_msg_remind_include_invalid",
        message_type="text",
        message="Airflow dingding text message remind users including invalid",
        # 123 is invalid user or user not in the group
        at_mobiles=["156XXXXXXXX", "123"],
        at_all=False,
    )

    # [START howto_operator_dingding_remind_users]
    text_msg_remind_all = DingdingOperator(
        task_id="text_msg_remind_all",
        message_type="text",
        message="Airflow dingding text message remind all users in group",
        # list of user phone/email here in the group
        # when at_all is specific will cover at_mobiles
        at_mobiles=["156XXXXXXXX", "130XXXXXXXX"],
        at_all=True,
    )
    # [END howto_operator_dingding_remind_users]

    link_msg = DingdingOperator(
        task_id="link_msg",
        message_type="link",
        message={
            "title": "Airflow dingding link message",
            "text": "Airflow official documentation link",
            "messageUrl": "https://airflow.apache.org",
            "picURL": "https://airflow.apache.org/_images/pin_large.png",
        },
    )

    # [START howto_operator_dingding_rich_text]
    markdown_msg = DingdingOperator(
        task_id="markdown_msg",
        message_type="markdown",
        message={
            "title": "Airflow dingding markdown message",
            "text": "# Markdown message title\n"
            "content content .. \n"
            "### sub-title\n"
            "![logo](https://airflow.apache.org/_images/pin_large.png)",
        },
        at_mobiles=["156XXXXXXXX"],
        at_all=False,
    )
    # [END howto_operator_dingding_rich_text]

    single_action_card_msg = DingdingOperator(
        task_id="single_action_card_msg",
        message_type="actionCard",
        message={
            "title": "Airflow dingding single actionCard message",
            "text": "Airflow dingding single actionCard message\n"
            "![logo](https://airflow.apache.org/_images/pin_large.png)\n"
            "This is a official logo in Airflow website.",
            "hideAvatar": "0",
            "btnOrientation": "0",
            "singleTitle": "read more",
            "singleURL": "https://airflow.apache.org",
        },
    )

    multi_action_card_msg = DingdingOperator(
        task_id="multi_action_card_msg",
        message_type="actionCard",
        message={
            "title": "Airflow dingding multi actionCard message",
            "text": "Airflow dingding multi actionCard message\n"
            "![logo](https://airflow.apache.org/_images/pin_large.png)\n"
            "Airflow documentation and GitHub",
            "hideAvatar": "0",
            "btnOrientation": "0",
            "btns": [
                {"title": "Airflow Documentation", "actionURL": "https://airflow.apache.org"},
                {"title": "Airflow GitHub", "actionURL": "https://github.com/apache/airflow"},
            ],
        },
    )

    feed_card_msg = DingdingOperator(
        task_id="feed_card_msg",
        message_type="feedCard",
        message={
            "links": [
                {
                    "title": "Airflow DAG feed card",
                    "messageURL": "https://airflow.apache.org/docs/apache-airflow/stable/ui.html",
                    "picURL": "https://airflow.apache.org/_images/dags.png",
                },
                {
                    "title": "Airflow grid feed card",
                    "messageURL": "https://airflow.apache.org/docs/apache-airflow/stable/ui.html",
                    "picURL": "https://airflow.apache.org/_images/grid.png",
                },
                {
                    "title": "Airflow graph feed card",
                    "messageURL": "https://airflow.apache.org/docs/apache-airflow/stable/ui.html",
                    "picURL": "https://airflow.apache.org/_images/graph.png",
                },
            ]
        },
    )

    msg_failure_callback = DingdingOperator(
        task_id="msg_failure_callback",
        message_type="not_support_msg_type",
        message="",
    )

    (
        [
            text_msg_remind_none,
            text_msg_remind_specific,
            text_msg_remind_include_invalid,
            text_msg_remind_all,
        ]
        >> link_msg
        >> markdown_msg
        >> [
            single_action_card_msg,
            multi_action_card_msg,
        ]
        >> feed_card_msg
        >> msg_failure_callback
    )

    from tests_common.test_utils.watcher import watcher

    # This test needs watcher in order to properly mark success/failure
    # when "tearDown" task with trigger rule is part of the DAG
    list(dag.tasks) >> watcher()

from tests_common.test_utils.system_tests import get_test_run  # noqa: E402

# Needed to run the example DAG with pytest (see: tests/system/README.md#run_via_pytest)
test_run = get_test_run(dag)



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from unittest import mock

from airflow.models.dag import DAG
from airflow.providers.dingding.operators.dingding import DingdingOperator
from airflow.utils import timezone

DEFAULT_DATE = timezone.datetime(2017, 1, 1)


class TestDingdingOperator:
    _config = {
        "dingding_conn_id": "dingding_default",
        "message_type": "text",
        "message": "Airflow dingding webhook test",
        "at_mobiles": ["123", "456"],
        "at_all": False,
    }

    def setup_method(self):
        args = {"owner": "airflow", "start_date": DEFAULT_DATE}
        self.dag = DAG("test_dag_id", schedule=None, default_args=args)

    @mock.patch("airflow.providers.dingding.operators.dingding.DingdingHook")
    def test_execute(self, mock_hook):
        operator = DingdingOperator(task_id="dingding_task", dag=self.dag, **self._config)

        assert operator is not None
        assert self._config["dingding_conn_id"] == operator.dingding_conn_id
        assert self._config["message_type"] == operator.message_type
        assert self._config["message"] == operator.message
        assert self._config["at_mobiles"] == operator.at_mobiles
        assert self._config["at_all"] == operator.at_all

        operator.execute(None)
        mock_hook.assert_called_once_with(
            self._config["dingding_conn_id"],
            self._config["message_type"],
            self._config["message"],
            self._config["at_mobiles"],
            self._config["at_all"],
        )
        mock_hook.return_value.send.assert_called_once_with()



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from collections.abc import Sequence
from typing import TYPE_CHECKING

from airflow.providers.common.compat.sdk import BaseOperator
from airflow.providers.dingding.hooks.dingding import DingdingHook

if TYPE_CHECKING:
    from airflow.providers.common.compat.sdk import Context


class DingdingOperator(BaseOperator):
    """
    This operator allows to send DingTalk message using Custom Robot API.

    .. seealso::
        For more information on how to use this operator, take a look at the guide:
        :ref:`howto/operator:DingdingOperator`

    :param dingding_conn_id: Dingding connection id that has access token in the password field,
        and optional host name in host field, if host not set than default
        ``https://oapi.dingtalk.com`` will use.
    :param message_type: Message type you want to send to Dingding, support five type so far
        including ``text``, ``link``, ``markdown``, ``actionCard``, ``feedCard``.
    :param message: The message send to chat group
    :param at_mobiles: Remind specific users with this message
    :param at_all: Remind all people in group or not. If True, will overwrite ``at_mobiles``
    """

    template_fields: Sequence[str] = ("message",)
    ui_color = "#4ea4d4"  # Operator icon color

    def __init__(
        self,
        *,
        dingding_conn_id: str = "dingding_default",
        message_type: str = "text",
        message: str | dict | None = None,
        at_mobiles: list[str] | None = None,
        at_all: bool = False,
        **kwargs,
    ) -> None:
        super().__init__(**kwargs)
        self.dingding_conn_id = dingding_conn_id
        self.message_type = message_type
        self.message = message
        self.at_mobiles = at_mobiles
        self.at_all = at_all

    def execute(self, context: Context) -> None:
        self.log.info("Sending Dingding message.")
        hook = DingdingHook(
            self.dingding_conn_id, self.message_type, self.message, self.at_mobiles, self.at_all
        )
        hook.send()
